"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const netatmo_1 = __importDefault(require("netatmo"));
let hap;
var VirtualRainSensorDeviceType;
(function (VirtualRainSensorDeviceType) {
    VirtualRainSensorDeviceType[VirtualRainSensorDeviceType["Switch"] = 0] = "Switch";
    VirtualRainSensorDeviceType[VirtualRainSensorDeviceType["Leak"] = 1] = "Leak";
})(VirtualRainSensorDeviceType || (VirtualRainSensorDeviceType = {}));
class VirtualRainSensorPlugin {
    constructor(logging, accessoryConfig) {
        this.logging = logging;
        this.accessoryConfig = accessoryConfig;
        this.netatmoStationId = undefined;
        this.netatmoRainSensorId = undefined;
        this.rainDetected = false;
        this.deviceType = this.initializeDeviceType(accessoryConfig.deviceType);
        this.virtualRainSensorService = this.createAndConfigureService(this.deviceType);
        this.pollingIntervalInSec = accessoryConfig.pollingInterval;
        this.slidingWindowSizeInMinutes = accessoryConfig.slidingWindowSize;
        this.cooldownIntervalInMinutes = accessoryConfig.cooldownInterval;
        this.rainSensorSerial = accessoryConfig.rainSensorSerial;
        this.IsInCooldown = false;
        // Reauthenticate Netatmo API every 24 hours
        this.reauthenticationIntervalInMs = 24 * 60 * 60 * 1000;
        // Create a new Accessory Information Service
        this.accessoryInformationService = new hap.Service.AccessoryInformation()
            .setCharacteristic(hap.Characteristic.Manufacturer, 'Patrick Bär')
            .setCharacteristic(hap.Characteristic.Model, 'Virtual Device for Netatmo Rain Sensor');
        this.logging.info('Authenticating with the Netatmo API and configuring callbacks.');
        this.netatmoApi = this.authenticateAndConfigureNetatmoApi(accessoryConfig);
        this.logging.info('Looking for Netatmo Rain Sensor and setting up polling schedule.');
        this.netatmoApi.getStationsData();
    }
    createAndConfigureService(deviceType) {
        let localVirtualRainSensorService;
        if (deviceType === VirtualRainSensorDeviceType.Switch) {
            // Create a new Switch Sensor Service
            localVirtualRainSensorService = new hap.Service.Switch(this.accessoryConfig.name);
            // Create handler for rain detection
            localVirtualRainSensorService.getCharacteristic(hap.Characteristic.On)
                .onGet(this.handleSwitchOnGet.bind(this));
            localVirtualRainSensorService.getCharacteristic(hap.Characteristic.On)
                .onSet(this.handleSwitchOnSet.bind(this));
            this.logging.info('Exposing the Netatmo Rain Sensor as a Switch.');
        }
        else {
            // Create a new Leak Sensor Service
            localVirtualRainSensorService = new hap.Service.LeakSensor(this.accessoryConfig.name);
            // Create handler for leak detection
            localVirtualRainSensorService.getCharacteristic(hap.Characteristic.LeakDetected)
                .onGet(this.handleLeakDetectedGet.bind(this));
            this.logging.info('Exposing the Netatmo Rain Sensor as a Leak Sensor.');
        }
        return localVirtualRainSensorService;
    }
    initializeDeviceType(deviceTypeAsString) {
        if (!deviceTypeAsString || deviceTypeAsString === '' || deviceTypeAsString === 'Switch') {
            return VirtualRainSensorDeviceType.Switch;
        }
        else {
            return VirtualRainSensorDeviceType.Leak;
        }
    }
    getDevices(_error, devices) {
        // Search with serial number
        if (this.rainSensorSerial) {
            devices.forEach(device => {
                device.modules.forEach(module => {
                    if (module.type === 'NAModule3') {
                        let currentRainSensorSerial = module._id;
                        currentRainSensorSerial = currentRainSensorSerial.substring(9);
                        currentRainSensorSerial = currentRainSensorSerial.replace(RegExp(':', 'g'), '');
                        // eslint-disable-next-line max-len
                        this.logging.debug(`Found Netatmo Rain Sensor named "${module.module_name}" with partial serial number "${currentRainSensorSerial}".`);
                        // Ignore first char for comparison
                        if (this.rainSensorSerial.substring(1) === currentRainSensorSerial) {
                            // eslint-disable-next-line max-len
                            this.logging.info(`Found Netatmo Rain Sensor named "${module.module_name}" with serial number "${this.rainSensorSerial}". Using this Rain Sensor.`);
                            this.netatmoStationId = device._id;
                            this.netatmoRainSensorId = module._id;
                        }
                    }
                });
            });
            if (!this.netatmoRainSensorId) {
                // eslint-disable-next-line max-len
                this.logging.warn(`Found no Netatmo Rain Sensor with serial number "${this.rainSensorSerial}". Trying to find default Rain Sensor.`);
            }
        }
        // Default rain sensor search in case no serial or no rain sensor found with serial
        if (!this.netatmoRainSensorId) {
            devices.forEach(device => {
                device.modules.forEach(module => {
                    if (module.type === 'NAModule3') {
                        this.logging.info(`Found Netatmo Rain Sensor named "${module.module_name}". Using this Rain Sensor.`);
                        this.netatmoStationId = device._id;
                        this.netatmoRainSensorId = module._id;
                    }
                });
            });
        }
        if (this.netatmoRainSensorId !== undefined) {
            // Create recurring timer for Netatmo API reauthentication
            setInterval(this.forceReauthenticationOfNetatmoApi.bind(this), this.reauthenticationIntervalInMs);
            // Create recurring timer for Netatmo API polling
            const pollingIntervalInMs = this.pollingIntervalInSec * 1000;
            this.logging.debug(`Setting Netatmo API polling interval to ${pollingIntervalInMs}ms.`);
            setInterval(this.pollNetatmoApi.bind(this), pollingIntervalInMs);
            this.pollNetatmoApi();
        }
        else {
            this.logging.error('No Netatmo Rain Sensor found.');
        }
    }
    getMeasures(_error, measures) {
        this.logging.debug(`Native output of measures: ${JSON.stringify(measures)}.`);
        if (this.IsInCooldown) {
            this.logging.debug('Cooldown detected. Skipping this rain detection cycle.');
        }
        else {
            this.rainDetected = false;
            measures.forEach(measure => {
                measure.value.forEach(measuredValue => {
                    if (measuredValue > 0) {
                        this.rainDetected = true;
                    }
                });
            });
            if (this.deviceType === VirtualRainSensorDeviceType.Switch) {
                this.handleSwitchOnSet(this.handleSwitchOnGet());
            }
            else {
                this.virtualRainSensorService.updateCharacteristic(hap.Characteristic.LeakDetected, this.handleLeakDetectedGet());
            }
        }
    }
    endCooldown() {
        this.IsInCooldown = false;
        this.logging.debug('Cooldown ended');
    }
    forceReauthenticationOfNetatmoApi() {
        this.logging.debug('Reauthenticating Netatmo API');
        this.shutdownNetatmoApi(this.netatmoApi);
        this.netatmoApi = this.authenticateAndConfigureNetatmoApi(this.accessoryConfig);
    }
    shutdownNetatmoApi(netatmoApi) {
        netatmoApi.removeAllListeners();
    }
    authenticateAndConfigureNetatmoApi(accessoryConfig) {
        const auth = {
            'client_id': accessoryConfig.netatmoClientId,
            'client_secret': accessoryConfig.netatmoClientSecret,
            'username': accessoryConfig.netatmoUsername,
            'password': accessoryConfig.netatmoPassword,
        };
        const netatmoApi = new netatmo_1.default(auth);
        netatmoApi.on('error', this.handleNetatmoApiError.bind(this));
        netatmoApi.on('warning', this.handleNetatmoApiWarning.bind(this));
        netatmoApi.on('get-stationsdata', this.getDevices.bind(this));
        netatmoApi.on('get-measure', this.getMeasures.bind(this));
        return netatmoApi;
    }
    handleNetatmoApiError(errorMessage) {
        this.logging.error(`Netatmo API error: ${errorMessage}.`);
    }
    handleNetatmoApiWarning(warningMessage) {
        this.logging.warn(`Netatmo API warning: ${warningMessage}.`);
    }
    pollNetatmoApi() {
        if (this.IsInCooldown) {
            this.logging.debug('Cooldown detected. Skipping this rain detection cycle.');
        }
        else {
            this.logging.debug('Polling the Netatmo API.');
            const now = new Date().getTime();
            const slidingWindowSizeInMillis = this.slidingWindowSizeInMinutes * 60 * 1000;
            this.logging.debug(`Sliding window size in milliseconds: ${slidingWindowSizeInMillis}.`);
            const options = {
                device_id: this.netatmoStationId,
                module_id: this.netatmoRainSensorId,
                scale: '30min',
                type: ['rain'],
                date_begin: Math.floor(new Date(now - slidingWindowSizeInMillis).getTime() / 1000),
                optimize: true,
                real_time: true,
            };
            this.logging.debug(`Native output of request options: ${JSON.stringify(options)}.`);
            this.netatmoApi.getMeasure(options);
        }
    }
    getServices() {
        return [
            this.accessoryInformationService,
            this.virtualRainSensorService,
        ];
    }
    scheduleSwitchReset() {
        if (this.deviceType === VirtualRainSensorDeviceType.Switch) {
            setTimeout(() => this.handleSwitchOnSet(false), 500);
        }
    }
    handleLeakDetectedGet() {
        if (this.handleSwitchOnGet()) {
            return hap.Characteristic.LeakDetected.LEAK_DETECTED;
        }
        else {
            return hap.Characteristic.LeakDetected.LEAK_NOT_DETECTED;
        }
    }
    handleSwitchOnGet() {
        if (this.IsInCooldown) {
            this.logging.debug('Rain detected is false during cooldown.');
            return false;
        }
        else {
            if (this.rainDetected) {
                this.logging.info('Rain detected!');
                if (this.cooldownIntervalInMinutes > 0) {
                    const cooldownIntervalInMs = this.cooldownIntervalInMinutes * 60 * 1000;
                    this.logging.debug(`Entering cooldown of ${cooldownIntervalInMs}ms.`);
                    this.IsInCooldown = true;
                    setTimeout(this.endCooldown.bind(this), cooldownIntervalInMs);
                }
                this.scheduleSwitchReset();
                return true;
            }
            else {
                this.logging.debug('No rain detected.');
                return false;
            }
        }
    }
    // value is type boolean for the switch service
    handleSwitchOnSet(value) {
        if (value) {
            this.logging.debug('Turn switch on.');
            this.scheduleSwitchReset();
        }
        else {
            this.logging.debug('Turn switch off.');
        }
        this.virtualRainSensorService.updateCharacteristic(hap.Characteristic.On, value);
    }
}
module.exports = (api) => {
    hap = api.hap;
    api.registerAccessory('homebridge-plugin-netatmo-rain-sensor', VirtualRainSensorPlugin);
};
//# sourceMappingURL=index.js.map